/*
 * PROJECT: Nachtblitz File Sharing Client
 * PROJECT-URL: http://echelon.i2p/nachtblitz/
 * LICENSE: GNU Affero General Public License, version 3 (AGPLv3)
 * FILE: Nachtblitz-1.4.26-src\Native\General.h
 * PURPOSE: Main header file
 * VERSION: 1.4.26
 * DATE: April 2, 2015
 * COPYRIGHT: Copyright (C) 2012-2015 Stephan A. Rger
 *            (nachtblitz@i2pmail.org). All rights reserved.
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of version 3 - no other version - of the
 * GNU Affero General Public License as published by the Free
 * Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License along with this program (agpl-3.0.txt in the parent
 * directory). If not, see <http://www.gnu.org/licenses/>.
 *
 *         __           _     _   _     _ _ _         _  _  _
 *      /\ \ \__ _  ___| |__ | |_| |__ | (_) |_ ____ / || || |
 *     /  \/ / _` |/ __| '_ \| __| '_ \| | | __|_  / | || || |_
 *    / /\  / (_| | (__| | | | |_| |_) | | | |_ / /  | ||__   _|
 *    \_\ \/ \__,_|\___|_| |_|\__|_.__/|_|_|\__/___| |_(_) |_|
 *
 */

#pragma once

#if defined(INCLUDE_WINDOWS)

#define WIN32_LEAN_AND_MEAN
// Require NT 5.0 (Windows 2000) at least.
#define WINVER          0x0500
#define _WIN32_WINNT    0x0500
#include <windows.h>
#include <wincrypt.h>

#endif // defined(INCLUDE_WINDOWS)

#include <assert.h>
#define ASSERT(x)       assert(x)

typedef unsigned __int8 BOOLEAN;

typedef signed __int8 SBYTE;
typedef signed __int16 INT16;
typedef signed __int32 INT32;
typedef signed __int64 INT64;

typedef unsigned __int8 BYTE;
typedef unsigned __int16 UINT16;
typedef unsigned __int32 UINT32;
typedef unsigned __int64 UINT64;

#define MAX_BYTE            ((BYTE)0xFF)
#define MAX_UINT16          ((UINT16)0xFFFF)
#define MAX_UINT32          ((UINT32)~0UL)
#define MAX_UINT64          ((UINT64)~0ULL)

#define BITS_PER_BYTE       (8)
#define BITS_PER_UINT16     (16)
#define BITS_PER_UINT32     (32)
#define BITS_PER_UINT64     (64)

#if _M_X64

typedef INT64 SCPUWORD;
typedef UINT64 CPUWORD;
#define MAX_CPUWORD         MAX_UINT64
#define BITS_PER_CPUWORD    BITS_PER_UINT64

#define CALLCONV            __stdcall

#else // _M_X64

typedef INT32 SCPUWORD;
typedef UINT32 CPUWORD;
#define MAX_CPUWORD         MAX_UINT32
#define BITS_PER_CPUWORD    BITS_PER_UINT32

#define CALLCONV            __cdecl

#endif // _M_X64

// The EXPORT macro is just a placeholder.
#define EXPORT

#define UNUSED_STACK_ITEM(x)        ((void)x)

#define IN
// METAIN means that some form of initilization is expected to have
// happened already, e.g. a valid reference must be passed, but the
// actual value at the referenced address is not considered input.
#define METAIN
#define OUT
#define OPTIONAL

#undef FALSE
#undef TRUE
#undef NULL

#define FALSE           ((BOOLEAN)0)
#define TRUE            ((BOOLEAN)!0)
#define NULL            ((void *)0)

// 32-bit rotation.
#define ROT_L(x, bits)  (_rotl((x), (bits)))
#define ROT_R(x, bits)  (_rotr((x), (bits)))
#define ROT(x, bits)    (ROT_R((x), (bits) % BITS_PER_UINT32))

#define BYTE_SWAP16(x)  (byteswap_ushort((x)))
#define BYTE_SWAP32(x)  (_byteswap_ulong((x)))
#define BYTE_SWAP64(x)  (_byteswap_uint64((x)))

#define ALIGN64         __declspec(align(8))

typedef UINT32 NBNAT_RESULTS;
#define NBNAT_FAILURE           (0)
#define NBNAT_PENDING           (1)
#define NBNAT_SUCCESS           (2)

#define UtilZeroMem(ptr, byte_size)     memset((ptr), 0, (byte_size))

BOOLEAN
UtilTryAquireLock(IN OUT UINT32 volatile * const a_Lock);

void
UtilAquireLock(IN OUT UINT32 volatile * const a_Lock);

void
UtilReleaseLock(IN OUT UINT32 volatile * const a_Lock);

__forceinline void
UtilSecWipeBuffer(METAIN OUT void volatile * const a_Buffer,
                  IN CPUWORD const a_Length)
{
    // NOTE: We use a volatile pointer to prevent the compiler from
    // optimizing this routine out.

#if defined(INCLUDE_WINDOWS)
    SecureZeroMemory((void *)a_Buffer, a_Length);
#else // defined(INCLUDE_WINDOWS)

#if defined(_M_AMD64)
    __stosb((BYTE *)a_Buffer, 0, a_Length);
#else // defined(_M_AMD64)
    BYTE volatile * l_Ptr = &((BYTE volatile *)a_Buffer)[a_Length];

    for (; a_Buffer != l_Ptr--;)
    {
        *l_Ptr = 0;
    }
#endif // defined(_M_AMD64)

#endif // defined(INCLUDE_WINDOWS)
}

__forceinline void
UtilSecWipeUInt32(METAIN OUT UINT32 volatile * const a_UInt32)
{
    *a_UInt32 = 0;
}

__forceinline void
UtilSecWipeUInt64(METAIN OUT UINT64 volatile * const a_UInt64)
{
    *a_UInt64 = 0;
}

__forceinline void
UtilSecWipePointer(METAIN OUT void const * volatile * const a_Ptr)
{
    *a_Ptr = NULL;
}

UINT32
UtilUInt64ToUInt32(IN UINT64 const a_Num);

UINT32
UtilUInt64ToUInt32AndRotate(IN UINT64 const a_Num,
                            IN UINT32 const a_Rotate);

UINT32
UtilPtrToUInt32(IN void const * const a_Ptr,
                IN UINT32 const a_Rotate);

// End of file.
